# -*- tcl -*-
#
# $Id: uninstall.tcl,v 1.16 2000/01/25 18:04:17 elias Exp $
# Copyright 1999 Security Dynamics
#
# Description:
#   Connect Agent Module uninstall script.
#
# Usage:
#	uninstall [-y] [ modules ]
#
#	Module names may be wildcard patterns.  Patterns will be
#	expanded with a trailing "*".  By default a list of all
#	installed modules is presented.
#
# 
#
set APPDIR {<<INSTALL:APPDIR>>}
set VARDIR {<<INSTALL:VARDIR>>}
set ETCDIR {<<INSTALL:ETCDIR>>}
set env(PATH) [file nativename $APPDIR/bin:$env(PATH)]

set lib [file join $env(TCL_LIBRARY)]

source $lib/instutils.tcl
source $lib/debug.tcl
source $lib/getopt.tcl
source $lib/info.tcl
source $lib/confutils.tcl
 

catch {namespace import CatInst::*}
catch {namespace import CatDbg::*}

set line70 {}
for {set i 0} {$i < 70} {incr i} { append line70 - }

#----------------------------------------------------------------------
# Initialize
#

foreach dir {APPDIR ETCDIR VARDIR} {
    if ![InstDir $dir [set $dir] msg] {
	puts stderr [lindex $msg 1]
	exit 1
    }
}

# Get args
#
set dont_ask 0
set allmodules 0
set opts(y_v) dont_ask
set opts(D_V) dbglvl
set opts(a_v) allmodules
getopt $argv "ayD:" errmsg opts
if [string length $errmsg] {
    puts stderr "$errmsg"
    puts stderr {Usage: uninstall [-y] [ module-or-pattern ... ]}
    exit 1
}
if $allmodules {
    set argv *
} else {
    set argv $opts(args)
}

# No modules, get a list, and pick one.
#
if {[llength $argv] == 0} {
    set list [ListInstalled info *]
    if {[llength $list] == 0} {
	puts stderr "Nothing installed?"
	exit 1
    }

    while 1 {
	puts stderr "Select module to uninstall:\n$line70"
	set i 0
	foreach m $list {
	    incr i
	    if [info exists info($m,description)] {
		puts stderr [format "%3d. %-15s %s" $i $m $info($m,description)]
	    } else {
		puts stderr [format "%3d. %s" $i $m]
	    }
	}
	puts stderr $line70
	set s {}
	if $dont_ask { set s {[a] }}
	puts -nonewline stderr "Choice (<number>,a = all,q = quit): $s"
	gets stdin answer
	set answer [string trim $answer]
	if [string match {[aA]*} $answer] {
	    break
	}
	if {$dont_ask && ![string length $answer]} {
	    break
	}
	if [string match {[qQ]*} $answer] {
	    exit 0
	}
	if {[scan $answer %d idx] == 1} {
	    if {$idx > 0 && $idx <= $i} {
		incr idx -1
		set list [lindex $list $idx]
		break
	    }
	}
    }
} else {
    set list {}
    foreach arg $argv {
	set list [concat $list [ListInstalled info $arg*]]
    }
    if {[llength $list] == 0} {
	puts stderr "No matching modules found"
    }
    if !$dont_ask {
	foreach m $list {
	    set info($m,kill) 0
	}
	puts stderr "Will uninstall the following modules:"
	puts stderr $line70
	foreach m $list {
	    if !$info($m,kill) {
		set info($m,kill) 1
		puts stderr [format "  %-15s %s" $m $info($m,description)]
	    }
	}
	puts stderr $line70
	while 1 {
	    puts -nonewline stderr "\nContinue? (y/n) \[y\]"
	    gets stdin answer
	    set answer [string trim $answer]
	    switch -glob -- $answer {
		{[yY]} {
		    break
		}
		"" {
		    break
		}
		{[nN]} {
		    puts stderr "Uninstallation aborted"
		    exit 0
		}
		default {
		    puts stderr "\nPlease answer \"y\" or \"n\"\n"
		}
	    }
	}
    }
}

# Get the required module(s) last
#
set tmplist {}
foreach m $list {
    if [info exists info($m,required)] {
	foreach r $info($m,required) {
	    lappend tmplist $r $m
	}
    } else {
	lappend tmplist $m {}
    }
}
set newlist {}
foreach m [dsort $tmplist] {
    if {[lsearch $list $m] > -1} {
	lappend newlist $m
    }
}
set list $newlist

# Write an uninstall log
#
if [info exists env(TEMP)] {
    set tmpdir $env(TEMP)
    set logfile $tmpdir/catuninstall.[pid]
}
if [info exists tmpdir] {
    cd $tmpdir
    DebugLogFile $logfile
    Log "#### [clock format [clock seconds]] ###"
    Log "Deinstalling $list"
}


# Uninstall!
#
CatInfo::Init $ETCDIR

foreach m $list {
    set info($m,already) 0
}
set code {}
set msg {}
foreach m $list {
    if $info($m,already) continue
    puts -nonewline stderr "Uninstalling $m..."
    if ![Uninstall $m err] {
	puts stderr "failed: [lindex $err 1]"
	exit 1
    }
    puts stderr "done"
    foreach {code msg} $err break
    if {[string compare $code PREUNINST] == 0} {
	puts $msg
    }
    set info($m,already) 1
}
#catch {file delete -force -- $logfile}
exit 0
