#
# $Id: distcopy.tcl,v 1.6 1999/09/14 12:40:59 perre Exp $
#

set DESCFILE description

if {[llength $argv] < 1} {
    puts stderr "Usage: $argv0 target-directory"
}
set TARGET_DIR [lindex $argv 0]/NT-5.0
if {[llength $argv] > 1} {
    set DESCFILE [lindex $argv 1]
}

proc warn { str } {
    puts stderr $str
}

#
# Takes a string composed as flags as arg.
# Known Flags are
#    d   Use destination name
#    e   Include external files
#
proc copy_file { dst src {flags {}} } {
    global TARGET_DIR MOD

    set td $TARGET_DIR/$MOD
    if ![file exists $td] {
	file mkdir $td
    }
    set from $src
    if ![file exists $src] {
	if [string match */version.txt [string tolower $dst]] {
	    create_version $td/[file tail $dst]
	    return
	}
	if [string match *.boks $dst] {
	    create_boks $TARGET_DIR/../adf/[file tail $dst]
	    return
	}
	set from [file tail $from]
    }
    set d [file dirname $src]
    if ![file exists $td/$d] {
	file mkdir $td/$d
    }
    if [string match *d* $flags] {
	puts stderr " + $src"
	file copy -force -- $from $td/$dst
    } else {
	puts stderr " + $src"
	file copy -force -- $from $td/$src
    }
    if [string match *e* $flags] {
	if [string match ../* $dst] {
	    file copy -force $from $td/[file tail $dst]
	}
    }
}

proc strip_file { dst src } {
    set in [open $src]
    set out [open $dst w]

    while {[gets $in s] > -1} {
	set s [string trim $s]
	if {[string length $s] == 0 || [string match #* $s]} continue
	puts $out $s
    }
    close $in
    close $out
}

# Put a number of lines into a file
#
proc print_to_file { file args } {
    set f [open $file w]

    foreach line $args {
	puts $f $line
    }
    close $f
}

# Create the information file for old security servers
#
proc create_boks { dst } {
    global MOD DESC CRYPT

    print_to_file $dst \
	    AGENT_ID=$MOD \
	    AGENT_DESCRIPTION=\"$DESC\" \
	    CRYPT_LICENSE=$CRYPT
}

# Create the informational VERSION.txt file
#
proc create_version { dst } {
    global DESC VERSION PATCHLEVEL REV CRYPT

    print_to_file $dst \
	    DESCRIPTION=\"$DESC\" \
	    VERSION=$VERSION \
	    PATCHLEVEL=$PATCHLEVEL \
	    REV=$REV \
	    CRYPT_LICENSE=$CRYPT
}

#----------------------------------------------------------------------
# Procs to parse the description file
#

set APPDIR {}
set VARDIR {}
set ETCDIR {}

proc fileset { module description args } {
    global MOD DESC TARGET_DIR

    set MOD $module
    set DESC $description
}


proc require { module } {
    global TARGET_DIR

    if ![file isdirectory $TARGET_DIR/$module] {
	warn "require: module \"$module\" is not in the distribution"
    }
}

proc version { version patchlevel revision crypto args } {
    global VERSION PATCHLEVEL REV CRYPT

    set VERSION $version
    set PATCHLEVEL $patchlevel
    set REV $revision
    set CRYPT $crypto
}

proc system { args } {
    set sys [lindex $args 0]
    if {[string compare $sys NT-5.0] != 0} {
	warn "system: expected NT-5.0, got \"$sys\""
    }
}

# These are ignored during packing
#
proc agent { args } { }
proc target { dirname args } { }
proc brand { args } { }
proc d { dirname args } { }
proc n { dirname args } { }
proc s { link target args } { }

# These just copy files
#
proc p { dst src args } { copy_file $dst $src }
proc P { dst src args } { copy_file $dst $src }
proc f { dst {src {}} args } { copy_file $dst $src }
proc F { dst src args } { copy_file $dst $src }
proc b { dst src args } { copy_file $dst $src }
proc B { dst src args } { copy_file $dst $src }
proc i { dst { src {}} args }	{ copy_file $dst $src de }

# Create adf/ dir if not present
#
if ![file exist $TARGET_DIR/../adf] {
    file mkdir $TARGET_DIR/../adf
}

# Read the description file as a tcl script
#
source $DESCFILE

# Copy the file itself, sans comments.
#
strip_file $DESCFILE.tmp $DESCFILE
file copy -force -- $DESCFILE.tmp $TARGET_DIR/$MOD/description

# Done
#
exit 0
