#
# $Id: rc.tcl,v 1.14 2000/05/12 07:10:54 elias Exp $
# Copyright 1999 Security Dynamics
#
# Add and remove startup rc-script.
# Variant:
#

namespace eval ::CatRC { ;#

catch {namespace import ::CatDbg::*}

variable appdir "<<INSTALL:APPDIR>>"
variable vardir "<<INSTALL:VARDIR>>"


variable rcdir "/usr/local/etc/rc.d"


proc Add_rc {script startname} {
   
     


    
    variable rcdir

    catch {Log "Installing rc-dir stype startup script"}

    if [file writable $rcdir] {
	if ![addentry $rcdir/$startname.sh $script] {
	    return 0
	}
    } else {
	catch {Log "Don't have permission to write $rcdir"}
    }
    

    
    
    
    
    
    

    return 1
}

proc Remove_rc {startname} {
    
     
    

    
    variable rcdir

    catch {Log "Removing rc-style sstartup script"}

    if [file writable $rcdir] {
	delentry $rcdir/$startname.sh
    } else {
	catch {Log "Don't have permission to write $rcdir"}
    }
     


    

    return 1
}


# Initialize or add entry to startup script
#
proc addentry {filename script} {
    variable appdir

    # Create a new file
    #
    if ![file exists $filename] {
	if [catch {file copy -- $script $filename} m] {
	    catch {Log "failed to copy rc-script; $m"}
	    return 0
	}
	catch {file attributes $filename -permissions 0744}
    }

    # Add an new entry to existing file
    #
    set startprg "$appdir/bin/start"
    set tmpfile "$filename.[pid]"
    if [catch {file rename -force $filename $tmpfile} error] {
	Log {Log "Cannot move $filename: $error"}
	return 0
    }
    if [catch {open $tmpfile {RDONLY}} fileId] {
	catch {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }
    set acc ""
    set done 0
    set inside 0
    while {[gets $fileId line] >= 0} {
	if !$done {
	    if $inside {
		if {[string compare $line "\""] == 0} {
		    set line "$startprg\n$line"
		    set done 1
		} else {
		    if {[string compare $line $startprg] == 0} {
			catch {Log "Current entry already present."}
			set done 1
		    }
		}
	    } else {
		if [regexp {^START=\"} $line] {
		    set inside 1
		}
	    }
	}
	append acc "$line\n"
    }
    close $fileId

    if [catch {open $tmpfile {WRONLY TRUNC}} fileId] {
	Log {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }

    puts -nonewline $fileId $acc
    close $fileId
    if [catch {file rename -force $tmpfile $filename} error] {
	Log {Log "Cannot move $tmpfile; $error"}
	catch {file delete -force -- $tmpfile}
	return 0
    }
    catch {Log "Adding starup entry"}
    catch {file attributes $filename -permissions 0744}

    return 1
}

# Remove entry from startup script, delete script when all entries are
# gone.
#
proc delentry {filename} {

    variable appdir

    if ![file exists $filename] {
	catch {Log "rc file $filename does not exist"}
	return 0
    }
    if ![file writable $filename] {
	catch {Log "rc file $filename is not writable"}
	return 0
    }

    set tmpfile $filename.[pid]
    set startprg "$appdir/bin/start"

    if [catch [file copy $filename $tmpfile] error] {
	catch {Log "Cannot copy $filename; $error"}
	return 0
    }

    if [catch {open $tmpfile {RDONLY}} fileId] {
	catch {Log "Cannot open $tmpfile"}
	catch {file delete -force -- $tmpfile}
	return 0
    }

    set acc ""
    set done 0
    set inside 0
    set nentries 0
    while {[gets $fileId line] >= 0} {
	if !$done {
	    if $inside {
		if {[string compare $line $startprg] == 0} {
		    set line ""
		} elseif {[string compare $line "\""] == 0} {
		    set done 1
		} else {
		    incr nentries
		}
	    } else {
		if [regexp {^START=\"} $line] {
		    set inside 1
		}
	    }
	}
	if {[string compare $line ""] != 0} {
	    append acc "$line\n"
	}
    }

    # When all entries are deleted, remove file
    #
    if {$nentries == 0} {
	close $fileId
	catch {file delete -force -- $tmpfile}
	catch {file delete -force -- $filename}
    } else {
	close $fileId
	if [catch {open $tmpfile {WRONLY TRUNC}} fileId] {
	    Log {Log "Cannot open $tmpfile"}
	    catch {file delete -force -- $tmpfile}
	    
	    return 0
	}
	puts -nonewline $fileId $acc
	close $fileId
	if [catch {file rename -force $tmpfile $filename} error] {
	    Log {Log "Cannot move $tmpfile; $error"}
	    catch {file delete -force -- $tmpfile}

	    return 0
	}
    }
    return 1
}


}; # end namespace CatRC
