!
! $Id: platform.pp,v 1.12 2000/03/16 15:10:32 mikko Exp $
! Copyright 2000 RSA Security
!
! System definitions
!
!   This file should be included in no-output mode.  There should be
!   nothing in here that depends on ppmk.pp.
!
! Exported symbols:
!
!   UNIX		: If defined, system is some kind of UNIX
!   WIN32		: If defined, system is Windows NT
!
!   CAT_{APP,VAR,ETC}DIR: SDK installation locations
!   BRAND		: SDK Brandname
!   EXPORT		: 0/1 If the crypto type is "exportable"
!   CRYPT_LICENSE	: doestic/export (as EXPORT, byt as a string)
!
!   SYSTEM		: Platform (according to the SDK)
!   COMPILER_FLAGS	: Mandatory flags for the C compiler
!   LINKER_FLAGS	: Mandatory flags for the linker
!   EXPORT_CFLAGS	: Crypto export flags (must be used)
!   PP_FLAGS		: Mandatory flags for the "pp" preprocessor
!   RC_FLAGS		: The resource compiler on NT
!
!   LIB_NET		: Network library flags, if any
!   LIB_THREAD		: Threading library flags, if any
!   LIB_NOTHREAD	: Thread replacement library, if any
!
!   LIB_CAT		: SDK base library, requires NET and THREAD/NOTHREAD
!   LIB_CATGEN		: SDK high-level library, requires CAT
!
!   EXE			: ".exe" suffix, if any, including the dot
!   OBJ			: "obj" suffix, _not_ including the dot
!   /			: Directory separator
!

@define(COMPILER_FLAGS,)
@define(LINKER_FLAGS,)
@define(SYSTEM, <<INSTALL:SYSTEM>>)
@define(PP_FLAGS, -DSYSTEM=@SYSTEM)
@define(CRYPT_LICENSE, <<INSTALL:CRYPT_LICENSE>>)
@define(EXPORT, <<INSTALL:EXPORT>>)

@define(CAT_VERSION, <<INSTALL:CATVERSION>>)

@define(CAT_APPDIR, <<INSTALL:APPDIR>>)
@define(CAT_VARDIR, <<INSTALL:VARDIR>>)
@define(CAT_ETCDIR, <<INSTALL:ETCDIR>>)

@define(LIB_THREAD,)
@define(LIB_NET,)
@define(LIB_CATGEN,)
@define(LIB_NOTHREAD,)

@define(PP, pp)

@define(EXPORT_CFLAGS, @if(@EXPORT, -DCAT_EXPORTABLE_CRYPTO))

! Match substring. Returns 1 if <long> string begins with <short>.
!
@function(_match, short, long)
@  if(@substr(0, @length(@short), @long) == @short, 1, 0)@;
@endfun

@if(@_match(NT-, @SYSTEM))
  @define(WIN32, 1)
  @undef(UNIX)
  @define(PP_FLAGS, -E @PP_FLAGS -I "@CAT_APPDIR\lib\ppmk")

  @define(CC, cl)
  @define(COMPILER_FLAGS, /nologo /MD /GF /O2 /DWIN32)
  @define(RC, rc)

  @define(RC_FLAGS,)
  @define(LINKER_FLAGS, /nologo)
  @define(LINKER, link)
@else
  @define(UNIX, 1)
  @undef(WIN32)
@endif

! Generic UNIX defines
!
@ifdef(UNIX)
  @define(OBJ, o)
  @define(EXE, )
  @define(/, /)
  @define(LIB_THREAD, -lpthread)
  @define(LIB_NOTHREAD, -lnothread)
  @define(PP_FLAGS, -E @PP_FLAGS -I @CAT_APPDIR/lib/ppmk)
  @define(CC, cc)
  @define(LINKER, cc)
@endif

! Win32 defines
!
@ifdef(WIN32)
  @define(OBJ, obj)
  @define(EXE, .exe)
  @define(/, \)
  @define(LIB_NET, wsock32.lib)
@endif

! Solaris 2.6
!
@if(@_match(solaris-2, @SYSTEM))
  @define(LIB_NET, -lsocket -lnsl)
  @define(LIB_NOTHREAD,)
@endif

! RedHat Linux 6.1
!
@if(@_match(redhat-6, @SYSTEM))
@endif

! IBM AIX 4.3
!
@if(@_match(aix-4, @SYSTEM))
  @define(LINKER_FLAGS, -brtl -bnolibpath)
  @define(COMPILER_FLAGS, -D_THREAD_SAFE)
@endif

! HP-UX 11.0
!
@if(@_match(hpux-11, @SYSTEM))
  @define(LINKER_FLAGS, @<-Wl,+s>@)
@endif

! FreeBSD 3.4 (should work on 4.0 too).
! Does not support threading (not the native user space threads, nor the
! port of linux threads) yet.  Besides, the user space threads are no
! performance gain.
!
@if(@_match(freebsd-3, @SYSTEM) || @_match(freebsd-4.0, @SYSTEM)) 
  @define(LIB_THREAD, )
  @define(LIB_NOTHREAD, )
@endif

! Agent libraries and flags
!
@ifdef(WIN32)
  @define(LIB_CAT, /LIBPATH:"@CAT_APPDIR\lib" catinit.lib cat.lib)
  @define(LIB_CATGEN, /LIBPATH:"@CAT_APPDIR\lib" catgen.lib)
  @define(CATFLAGS, @EXPORT_CFLAGS -I "@CAT_APPDIR\include")
@else 
  @define(LIB_CAT, -L@CAT_APPDIR/lib -lcatinit -lcat)
  @define(LIB_CATGEN, -L@CAT_APPDIR/lib -lcatgen)
  @define(CATFLAGS, @EXPORT_CFLAGS -I @CAT_APPDIR/include)
@endif

! Customization: $PPMKRC, ~/.ppmk
!
@ifdef(UNIX)
  @include(-noerror, -nooutput, -or,
		     @+PPMKRC, @+HOME/.ppmk)
@else
  @include(-noerror, -nooutput, @+PPMKRC)
@endif





