!
! $Id: ppmk.pp,v 1.7 2000/01/26 15:17:27 perre Exp $
! Copyright 2000 RSA Security
!
! Function definitions used by "ppmk".  This file should be included
! in no-output mode.  The  comment-like text starting with "!" are
! actually not comments at all, as far as the "pp" preprocessor
! is concerned.
!

!======================================================================
! Public functions
!
! TARGETS(target list)
!
!  Define the list of default targets.  Also includes the makefile
!  template.  This should be the first function called, after any
!  definitions.
!
!
! PROGRAM(name, c-files, libraries and linker flags [, depend ])
!
!  Produce a program target.  Avoid using explicit linker flags and
!  libraries -- stick to using symbols.  The name of the program
!  should not include any ".exe" suffix.
!
!
! COMPILEFLAGS(c-files, flags [, depend ])
!
!  Produce specific targets to build object files from c-files,
!  with extra flags in addition to the ones defined in CFLAGS.
!
!
! PPFILE(to, from [, pp-flags [, depend ]])
!
!  Produce a target that preprocesses some file.  Will include all
!  symbols in "platform.pp"
!
!
! VERSIONFILE(filename, headerfile [, pp-flags ])
!
!  Produce a version headerfile from a pp-definition file.
!  Will only use the definitions in the file. If building on
!  WIN32 it will automatically try and compile a Resource
!  file containing the information available in the Version tab
!  in the binary files Properties window.
!
!======================================================================

@function(TARGETS, targets)
@  define(_targets,  $(VERSION_H) @_flat(@targets))
@  ifdef(WIN32)
@    _append(_targets, $(RC_RES))
@  endif
@  define(PPMK_TARGETS,1)
@  include(ppmk.tmpl)
@  undef(PPMK_TARGETS)
@endfun

@function(PROGRAM, name, cfiles, ...)
@  local(objs)
@  define(objs, @_fix(, .@OBJ, @_nosuffix(@cfiles)))
@  _append(_objs, @objs)
@  _append(_cleanupfiles, @name@EXE @objs)

@  ifdef(WIN32)
@     define(objs, $(RC_RES) @objs)
@  endif


# PROGRAM @name
#
@if(@EXE != "")
@name: @name@EXE

@endif
@name@EXE: @_flat(@2) @objs
	$(LINKER) $(LDFLAGS) $(LINKOUT)$@@  @objs @_flat(@1)

#
@endfun

@function(COMPILEFLAGS, files, flags, ...)
@  local(o)
@  splitfor(f, @files)
@    define(o, @_basename(@_nosuffix(@f)))

# COMPILEFLAGS @o.c
#
@o.@OBJ: @f @_flat(@1) $(HEADERS)
	$(CC) -c @f -o $@@ $(CFLAGS) @_flat(@flags)

#
@  endfor
@endfun

@function(VERSIONFILE, filename, headerfile, ...)
@  _append(_cleanupfiles, $(VERSION_H))

VERSION_H = @headerfile

# VERSIONFILE
#
$(VERSION_H): @filename
	$(PP) -NScpp -i $? @_flat(@1) > $@@

@ifdef(WIN32)
@  _append(_cleanupfiles, $(RC_INFO) $(RC_RES))

RC_INFO = version.rc

# RC Version info.
#
$(RC_INFO): @filename
	$(PP) -E -DSYSTEM=NT-4.0 -o $@@ -i $? "$(CATLIBDIR)\ppmk\ppmk_rc.tmpl"

RC_RES = version.res

# Compile the resource info with the resource compiler.
#
$(RC_RES): $(RC_INFO)
	$(RC) /fo $@@ /r $?
 
@endif 
@endfun

@function(PPFILE, to, from, ...)
@  _append(_cleanupfiles, @to)

# PROCFILE
#
@_flat(@to): @_flat(@2) @from
	$(PP) $(PPFLAGS) -o $@@ @_flat(@1) @from

@endfun


!======================================================================
! Help functions.  Private.
!

! _pathcat(path, path, path ...) ---
!
! Concatenate path components for use in a makefile.  Looks like
! nmake, quotes and the windows prefernce for path names with
! spaces in them may result in all sorts of trouble.
!
@function(_pathcat, ...)
@  local(path)
@  ifdef(UNIX)
@    for(p, @*)
@      if(@p != "", @define(path, @path@if(@path != "", /)@p))
@    endfor
@  endif
@  ifdef(WIN32)
@    for(p, @*)
@      if(@p != "", @define(path, @path@if(@path != "", \)@p))
@    endfor
@  endif
@  path@
@endfun

! _nosuffix(file list) --
!
! Remove the last suffix, if any, from a list of file names.
!
@function(_nosuffix, files)
@  local(x, list, x1, file, result)
@  splitfor(file, @files)
@    splitfor(x, @file, .)
@      define(list, @list@if(@list != "",.)@x1)
@      define(x1, @x)
@    endfor
@    define(result, @result @if(@list == "",@x1,@list))
@    define(list,)
@    define(x1,)
@  endfor
@  substr(1, -0, @result)@
@endfun

! _fix(prefix, suffix, filename list) --
!
! Add prefixes and/or suffixes to all elements in a list (files)
!
@function(_fix, prefix, suffix, list)
@  local(x, y)
@  splitfor(x, @list)
@    define(y, @y @prefix@x@suffix)
@  endfor
@  y@
@endfun

! _flat(string list) --
!
! Replace all whitespace in a string with single spaces
!
@function(_flat, string)
@  local(x, result)
@  splitfor(x, @string)
@    define(result, @+result@if(@+result != "",@< >@)@x)
@  endfor
@  result@
@endfun

! _basename(filename list) --
!
! Strip directories off one or more files.
!
@function(_basename, files)
@  local(x, p, file, result)
@  splitfor(file, @files)
@    splitfor(p, @file, @(/))
@      define(x, @p)
@    endfor
@    define(result, @result @x)
@  endfor
@  substr(1, @length(@result), @result)@
@endfun

! Append to a variable, as a white-space-separated list
!
@function(_append, var, what)
@  define(@var, @ifdef(@var, @(@var) )@what)
@endfun

! Load system definitions.
!
@include(platform.pp)
